/*
 * \file uart.c
 *
 * \date Feb 16, 2017
 * \author Ting
 */

#include "uart.h"

/*!
 * \brief Inicializa os pinos conectados a UART0 e ao OpenSDA.
 * \param[in] taxa taxa de amostragem
 * \param[in] baud_rate baud rate
 */
uint8_t initUART(uint8_t taxa, unsigned int baud_rate) {
	/*!
	 * Configura&ccedil;&atilde;o de SIM
         */
	SIM_CLKDIV1 &= (uint32_t)~(uint32_t)(SIM_CLKDIV1_OUTDIV4(xxxx));  ////< bus_clock = clock/1
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;
	SIM_SCGC4 |= SIM_SCGC4_UART0_MASK;
	SIM_SOPT2 |= SIM_SOPT2_UART0SRC(xxxx);	///< configura a fonte de relgio (20.971520MHz)
	SIM_SOPT2 &= ~SIM_SOPT2_PLLFLLSEL_MASK;	 
	                                         ///< no reset, o valor  zero (FLL)
	                                         ///< mas no deixa de ser uma boa
	                                         ///< prtica de inicializar explicitamente
	
	/*!
	 * Configura&ccedil;&atilde;o dos pinos que servem UART0 
     */
	PORTA_PCR1 |= PORT_PCR_MUX(xxxx);         ///< Configure pino com fun&ccedil;&atilde;o UART0_RX
	PORTA_PCR2 |= PORT_PCR_MUX(xxxx);         ///< Configure pino com fun&ccedil;&atilde;o UART0_TX
		
	/*!
	 * Configura&ccedil;&atilde;o do m&oacute;dulo UART0 
         */
	UART0_C1 &= ~(xxxx |			///< Configure "no paridade"
			UART0_C1_ILT_MASK |
			UART0_C1_WAKE_MASK |
			xxxx |                   ///< Configure "dados de 8 bits"
			UART0_C1_DOZEEN_MASK |
			UART0_C1_LOOPS_MASK );
	/*!
	 * Desabilita canal receptor e transmissor para configurar baud rate
	 */
	UART0_C2 &= ~(UART_C2_RE_MASK |           
				UART_C2_TE_MASK);
	UART0_BDH &= ~(xxxx |                   ///< Configure n&uacute;mero de stop bits: 1  
				UART_BDH_RXEDGIE_MASK | 
				UART_BDH_LBKDIE_MASK);
	
	UART0_C4 &= ~(UART0_C4_OSR(xxxx));           ///< Configura taxa de superamostragem 4
	/*!
	 * baud rate 19200 (Se&ccedil;&atilde;o 8.3.2 em 
	 * ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KLQRUG.pdf)
	 */
	UART0_BDH |= UART_BDH_SBR(xxxx);                             
	UART0_BDL |= UART_BDL_SBR(xxxx);             ///< Configure baud rate 4800
	
	
	UART0_S1 |= (UART0_S1_PF_MASK |              ///< Registradores de estado: w1c (limpar)
				UART0_S1_FE_MASK |
				UART0_S1_NF_MASK |
				UART0_S1_OR_MASK |
				UART0_S1_IDLE_MASK);
	
	UART0_C5 |= xxxx;       ///< amostra as duas bordas nos dados do receptor (entre taxa de amostragem 4x e 7x)

	UART0_C2 |= (xxxx |         		///< habilita o canal receptor 
		    xxxx);	  		///< habilita o canal transmissor            
	return 1;
}

/*!
 * \brief Habilita a interrup&ccedil;&atilde;o do canal receptor
 */
void enableRIEInterrup (void) {
}

/*!
 * \brief Habilita a interrup&ccedil;&atilde;o do canal transmissor
 */
void enableTEInterrup (void) {
}

/*!
 * \brief Desabilita a interrup&ccedil;&atilde;o do canal receptor
 */
void disableRIEInterrup (void) {
}

/*!
 * \brief Desabilita a interrup&ccedil;&atilde;o do canal transmissor
 */
void disableTEInterrup (void) {
}

/*!
 * \brief Habilita o canal receptor
 */
void enableRE (void) {
}

/*!
 * \brief Habilita o canal transmissor
 */
void enableTE (void) {
}

void disableRE (void) {
	UART0_C2 &= ~UART_C2_RE_MASK;
}

void disableTE (void) {
	UART0_C2 &= ~UART_C2_TE_MASK;
	
}
