/*!
 * \file main.c
 * \brief Comunica&ccedil;&atilde;o Serial Ass&iacute;ncrono UART.
 * \details Este c&oacute;digo testa a configura&ccedil;&atilde;o no modo de 
 *          interrup&ccedil;&atilde;o do m&oacute;dulo UARTx dispon&iacute;vel no MKZ25Z128. 
 *          &Eacute; demonstrado o envio de string de nome de cor de acordo com a letra digitdas.       
 * \mainpage Projeto de envio de nome de cores conforme a letra digitada.
 * \author Wu Shin-Ting
 * \date 19/09/2016
 * \note Documenta&ccedil;&atilde;o do c&oacute;digo no formato de Doxygen:
 *       http://mcuoneclipse.com/2014/09/01/automatic-documentation-generation-doxygen-with-processor-expert/
 */

#include "derivative.h" 
#include "ledRGB.h"
#include "uart.h"
#include "nvic.h"
#include "estrutura.h"

/*! 
 * vari&aacute;vel de controle da contagem de tempo (m&uacute;ltiplo de 0.25s)
 */
uint32_t tempo;       

/*!
 * vari&aacute;vel de cor selecionada
 */         
uint8_t indice;            

/*!
 * vari&aacute;vel de estado da botoeira PTA12 (1: pressionado; 0 solto)
 */
uint8_t estado_pta12;       

/*!
 * varivel de estado do ledRGB
 */
uint8_t apagado;

/*!
 * vetor de cores do led
 */
cor cores_estaticas[] = {{ACESO,APAGADO,APAGADO},{ACESO,ACESO,ACESO},{ACESO,ACESO,APAGADO},
				{APAGADO,ACESO,ACESO},{ACESO,APAGADO,ACESO},{APAGADO,APAGADO,APAGADO}};

/*!
 * buffers de dados de interface com UART
 */
buffer_circular buffer_receptor, buffer_transmissor;

#define TAM_MAX 1024

/*!
 * \attention O evento de interrup&ccedil;&atilde;o do canal receptor &eacute; a presen&ccedil;a de 
 * um novo caractere no registrador UARTx_D (de leitura) enquanto o evento que causa uma interrup&ccedil;&atilde;o 
 * no canal de transmissor &eacute; o registrador UARTx_D (de escrita) ficar ocioso. Ap&oacute;s
 * Reset, UARTx_D fica sempre ocioso. 
 */
int main(void)
{
	uint8_t i, j, ind;
    char d;
	char *nomes_cores[]={"VERMELHO","BRANCO","AMARELO","CIANO","MAGENTA","PRETO"};

	/*! 
	 * Inicialize os pinos PTA1 e PTA2 como UART0
	 */
	initUART(4,4800);   ///< Configure: taxa de amostragem 4x e baudrate = 4800
	enableRIEInterrup();
	enableNVICUART(1);
	
	/*!
	 * Inicializa&ccedil;&atilde;o das vari&aacute;veis
	 */
	ind = 5;
	bufCircular_cria (&buffer_receptor, TAM_MAX);
	bufCircular_cria (&buffer_transmissor, TAM_MAX);
	
	for(;;) {
		if (!bufCircular_vazia (&buffer_receptor)) {
			/*! Se o receptor tem algum caractere*/
			d = bufCircular_remove(&buffer_receptor);
			switch (d)
			{
			case 'V':
			case 'v':
				ind = 0;
				break;
			case 'B':
			case 'b':
				ind = 1;
				break;
			case 'A':
			case 'a':
				ind = 2;
				break;
			case 'C':
			case 'c':
				ind = 3;
				break;
			case 'M':
			case 'm':
				ind = 4;
				break;
			default:
				ind = 5;
			}	
		}
		/*!
		 * Envia nome de cor selecionada para terminal
		 */
		for (i=0; i<10; i++) {
			j = 0;
			while (nomes_cores[ind][j]) {
				while (bufCircular_cheia(&buffer_transmissor));   ///< espera vagar uma casa no buffer
				bufCircular_insere(&buffer_transmissor, nomes_cores[ind][j]);  
				enableTEInterrup();  ///< habilita a interrup&ccedil;&atilde;odo transmissor
				j++;
			}
			while (bufCircular_cheia(&buffer_transmissor));   ///< espera vagar uma casa no buffer
			bufCircular_insere(&buffer_transmissor, ' ');     ///< envia um espa&ccedil;o branco
		}
		while (bufCircular_cheia(&buffer_transmissor));   ///< espera vagar uma casa no buffer
		bufCircular_insere(&buffer_transmissor, '\r');    ///< envia caractere de controle CR (Carriage Return)
		while (bufCircular_cheia(&buffer_transmissor));   ///< espera vagar uma casa no buffer
		bufCircular_insere(&buffer_transmissor, '\n');    ///< envia caractere de controle LF (Line Feed)
	}
	/*!
	 * Liberar &aacute;rea na mem&oacute;ria
	 */
	bufCircular_deleta (&buffer_receptor);
	bufCircular_deleta (&buffer_transmissor);
}
